@@ -25,7 +25,7 @@ print(__doc__)
 
 from time import time
 
-import pylab as pl
+import matplotlib.pyplot as plt
 from mpl_toolkits.mplot3d import Axes3D
 from matplotlib.ticker import NullFormatter
 
@@ -39,18 +39,18 @@ X, color = datasets.samples_generator.make_s_curve(n_points, random_state=0)
 n_neighbors = 10
 n_components = 2
 
-fig = pl.figure(figsize=(15, 8))
-pl.suptitle("Manifold Learning with %i points, %i neighbors"
-            % (1000, n_neighbors), fontsize=14)
+fig = plt.figure(figsize=(15, 8))
+plt.suptitle("Manifold Learning with %i points, %i neighbors"
+             % (1000, n_neighbors), fontsize=14)
 
 try:
     # compatibility matplotlib < 1.0
-    ax = fig.add_subplot(241, projection='3d')
-    ax.scatter(X[:, 0], X[:, 1], X[:, 2], c=color, cmap=pl.cm.Spectral)
+    ax = fig.add_subplot(251, projection='3d')
+    ax.scatter(X[:, 0], X[:, 1], X[:, 2], c=color, cmap=plt.cm.Spectral)
     ax.view_init(4, -72)
 except:
-    ax = fig.add_subplot(241, projection='3d')
-    pl.scatter(X[:, 0], X[:, 2], c=color, cmap=pl.cm.Spectral)
+    ax = fig.add_subplot(251, projection='3d')
+    plt.scatter(X[:, 0], X[:, 2], c=color, cmap=plt.cm.Spectral)
 
 methods = ['standard', 'ltsa', 'hessian', 'modified']
 labels = ['LLE', 'LTSA', 'Hessian LLE', 'Modified LLE']
@@ -63,23 +63,23 @@ for i, method in enumerate(methods):
     t1 = time()
     print("%s: %.2g sec" % (methods[i], t1 - t0))
 
-    ax = fig.add_subplot(242 + i)
-    pl.scatter(Y[:, 0], Y[:, 1], c=color, cmap=pl.cm.Spectral)
-    pl.title("%s (%.2g sec)" % (labels[i], t1 - t0))
+    ax = fig.add_subplot(252 + i)
+    plt.scatter(Y[:, 0], Y[:, 1], c=color, cmap=plt.cm.Spectral)
+    plt.title("%s (%.2g sec)" % (labels[i], t1 - t0))
     ax.xaxis.set_major_formatter(NullFormatter())
     ax.yaxis.set_major_formatter(NullFormatter())
-    pl.axis('tight')
+    plt.axis('tight')
 
 t0 = time()
 Y = manifold.Isomap(n_neighbors, n_components).fit_transform(X)
 t1 = time()
 print("Isomap: %.2g sec" % (t1 - t0))
-ax = fig.add_subplot(246)
-pl.scatter(Y[:, 0], Y[:, 1], c=color, cmap=pl.cm.Spectral)
-pl.title("Isomap (%.2g sec)" % (t1 - t0))
+ax = fig.add_subplot(257)
+plt.scatter(Y[:, 0], Y[:, 1], c=color, cmap=plt.cm.Spectral)
+plt.title("Isomap (%.2g sec)" % (t1 - t0))
 ax.xaxis.set_major_formatter(NullFormatter())
 ax.yaxis.set_major_formatter(NullFormatter())
-pl.axis('tight')
+plt.axis('tight')
 
 
 t0 = time()
@@ -87,12 +87,12 @@ mds = manifold.MDS(n_components, max_iter=100, n_init=1)
 Y = mds.fit_transform(X)
 t1 = time()
 print("MDS: %.2g sec" % (t1 - t0))
-ax = fig.add_subplot(247)
-pl.scatter(Y[:, 0], Y[:, 1], c=color, cmap=pl.cm.Spectral)
-pl.title("MDS (%.2g sec)" % (t1 - t0))
+ax = fig.add_subplot(258)
+plt.scatter(Y[:, 0], Y[:, 1], c=color, cmap=plt.cm.Spectral)
+plt.title("MDS (%.2g sec)" % (t1 - t0))
 ax.xaxis.set_major_formatter(NullFormatter())
 ax.yaxis.set_major_formatter(NullFormatter())
-pl.axis('tight')
+plt.axis('tight')
 
 
 t0 = time()
@@ -101,11 +101,23 @@ se = manifold.SpectralEmbedding(n_components=n_components,
 Y = se.fit_transform(X)
 t1 = time()
 print("SpectralEmbedding: %.2g sec" % (t1 - t0))
-ax = fig.add_subplot(248)
-pl.scatter(Y[:, 0], Y[:, 1], c=color, cmap=pl.cm.Spectral)
-pl.title("SpectralEmbedding (%.2g sec)" % (t1 - t0))
+ax = fig.add_subplot(259)
+plt.scatter(Y[:, 0], Y[:, 1], c=color, cmap=plt.cm.Spectral)
+plt.title("SpectralEmbedding (%.2g sec)" % (t1 - t0))
 ax.xaxis.set_major_formatter(NullFormatter())
 ax.yaxis.set_major_formatter(NullFormatter())
-pl.axis('tight')
+plt.axis('tight')
 
-pl.show()
+t0 = time()
+tsne = manifold.TSNE(n_components=n_components, init='pca', random_state=0)
+Y = tsne.fit_transform(X)
+t1 = time()
+print("t-SNE: %.2g sec" % (t1 - t0))
+ax = fig.add_subplot(250)
+plt.scatter(Y[:, 0], Y[:, 1], c=color, cmap=plt.cm.Spectral)
+plt.title("t-SNE (%.2g sec)" % (t1 - t0))
+ax.xaxis.set_major_formatter(NullFormatter())
+ax.yaxis.set_major_formatter(NullFormatter())
+plt.axis('tight')
+
+plt.show()
