@@ -15,7 +15,7 @@ the training samples.
 print(__doc__)
 
 import numpy as np
-import pylab as pl
+import matplotlib.pyplot as plt
 
 from sklearn.datasets import load_iris
 from sklearn.tree import DecisionTreeClassifier
@@ -30,7 +30,7 @@ iris = load_iris()
 
 for pairidx, pair in enumerate([[0, 1], [0, 2], [0, 3],
                                 [1, 2], [1, 3], [2, 3]]):
-     # We only take the two corresponding features
+    # We only take the two corresponding features
     X = iris.data[:, pair]
     y = iris.target
 
@@ -50,7 +50,7 @@ for pairidx, pair in enumerate([[0, 1], [0, 2], [0, 3],
     clf = DecisionTreeClassifier().fit(X, y)
 
     # Plot the decision boundary
-    pl.subplot(2, 3, pairidx + 1)
+    plt.subplot(2, 3, pairidx + 1)
 
     x_min, x_max = X[:, 0].min() - 1, X[:, 0].max() + 1
     y_min, y_max = X[:, 1].min() - 1, X[:, 1].max() + 1
@@ -59,20 +59,20 @@ for pairidx, pair in enumerate([[0, 1], [0, 2], [0, 3],
 
     Z = clf.predict(np.c_[xx.ravel(), yy.ravel()])
     Z = Z.reshape(xx.shape)
-    cs = pl.contourf(xx, yy, Z, cmap=pl.cm.Paired)
+    cs = plt.contourf(xx, yy, Z, cmap=plt.cm.Paired)
 
-    pl.xlabel(iris.feature_names[pair[0]])
-    pl.ylabel(iris.feature_names[pair[1]])
-    pl.axis("tight")
+    plt.xlabel(iris.feature_names[pair[0]])
+    plt.ylabel(iris.feature_names[pair[1]])
+    plt.axis("tight")
 
     # Plot the training points
     for i, color in zip(range(n_classes), plot_colors):
         idx = np.where(y == i)
-        pl.scatter(X[idx, 0], X[idx, 1], c=color, label=iris.target_names[i],
-                   cmap=pl.cm.Paired)
+        plt.scatter(X[idx, 0], X[idx, 1], c=color, label=iris.target_names[i],
+                    cmap=plt.cm.Paired)
 
-    pl.axis("tight")
+    plt.axis("tight")
 
-pl.suptitle("Decision surface of a decision tree using paired features")
-pl.legend()
-pl.show()
+plt.suptitle("Decision surface of a decision tree using paired features")
+plt.legend()
+plt.show()
