@@ -1,4 +1,13 @@
-"""Meanshift clustering."""
+"""Mean shift clustering algorithm.
+
+Mean shift clustering aims to discover *blobs* in a smooth density of
+samples. It is a centroid based algorithm, which works by updating candidates
+for centroids to be the mean of the points within a given region. These
+candidates are then filtered in a post-processing stage to eliminate
+near-duplicates to form the final set of centroids.
+
+Seeding is performed using a binning technique for scalability.
+"""
 
 # Authors: Conrad Lee <conradlee@gmail.com>
 #          Alexandre Gramfort <alexandre.gramfort@inria.fr>
@@ -8,28 +17,32 @@ from collections import defaultdict
 import numpy as np
 
 from ..externals import six
-from ..utils import extmath, check_random_state
+from ..utils import extmath, check_random_state, gen_batches
 from ..base import BaseEstimator, ClusterMixin
 from ..neighbors import NearestNeighbors
+from ..metrics.pairwise import pairwise_distances_argmin
 
 
 def estimate_bandwidth(X, quantile=0.3, n_samples=None, random_state=0):
-    """Estimate the bandwidth to use with MeanShift algorithm
+    """Estimate the bandwidth to use with the mean-shift algorithm.
+
+    That this function takes time at least quadratic in n_samples. For large
+    datasets, it's wise to set that parameter to a small value.
 
     Parameters
     ----------
-    X : array [n_samples, n_features]
+    X : array-like, shape=[n_samples, n_features]
         Input points.
 
     quantile : float, default 0.3
         should be between [0, 1]
         0.5 means that the median of all pairwise distances is used.
 
-    n_samples : int
-        The number of samples to use. If None, all samples are used.
+    n_samples : int, optional
+        The number of samples to use. If not given, all samples are used.
 
     random_state : int or RandomState
-        Pseudo number generator state used for random sampling.
+        Pseudo-random number generator state used for random sampling.
 
     Returns
     -------
@@ -42,30 +55,34 @@ def estimate_bandwidth(X, quantile=0.3, n_samples=None, random_state=0):
         X = X[idx]
     nbrs = NearestNeighbors(n_neighbors=int(X.shape[0] * quantile))
     nbrs.fit(X)
-    d, _ = nbrs.kneighbors(X, return_distance=True)
 
-    bandwidth = np.mean(np.max(d, axis=1))
-    return bandwidth
+    bandwidth = 0.
+    for batch in gen_batches(len(X), 500):
+        d, _ = nbrs.kneighbors(X[batch, :], return_distance=True)
+        bandwidth += np.max(d, axis=1).sum()
+
+    return bandwidth / X.shape[0]
 
 
 def mean_shift(X, bandwidth=None, seeds=None, bin_seeding=False,
                min_bin_freq=1, cluster_all=True, max_iterations=300):
-    """Perform MeanShift Clustering of data using a flat kernel
-
-    Seed using a binning technique for scalability.
+    """Perform mean shift clustering of data using a flat kernel.
 
     Parameters
     ----------
 
-    X : array-like shape=[n_samples, n_features]
+    X : array-like, shape=[n_samples, n_features]
         Input data.
 
     bandwidth : float, optional
         Kernel bandwidth.
-        If bandwidth is not defined, it is set using
-        a heuristic given by the median of all pairwise distances.
 
-    seeds : array [n_seeds, n_features]
+        If bandwidth is not given, it is determined using a heuristic based on
+        the median of all pairwise distances. This will take quadratic time in
+        the number of samples. The sklearn.cluster.estimate_bandwidth function
+        can be used to do this more efficiently.
+
+    seeds : array-like, shape=[n_seeds, n_features]
         Point used as initial kernel locations.
 
     bin_seeding : boolean
@@ -84,10 +101,10 @@ def mean_shift(X, bandwidth=None, seeds=None, bin_seeding=False,
     Returns
     -------
 
-    cluster_centers : array [n_clusters, n_features]
+    cluster_centers : array, shape=[n_clusters, n_features]
         Coordinates of cluster centers.
 
-    labels : array [n_samples]
+    labels : array, shape=[n_samples]
         Cluster labels for each point.
 
     Notes
@@ -157,7 +174,7 @@ def mean_shift(X, bandwidth=None, seeds=None, bin_seeding=False,
 
 
 def get_bin_seeds(X, bin_size, min_bin_freq=1):
-    """Finds seeds for mean_shift
+    """Finds seeds for mean_shift.
 
     Finds seeds by first binning data onto a grid whose lines are
     spaced bin_size apart, and then choosing those bins with at least
@@ -200,21 +217,40 @@ def get_bin_seeds(X, bin_size, min_bin_freq=1):
 
 
 class MeanShift(BaseEstimator, ClusterMixin):
-    """MeanShift clustering
+    """Mean shift clustering using a flat kernel.
+
+    Mean shift clustering aims to discover "blobs" in a smooth density of
+    samples. It is a centroid-based algorithm, which works by updating
+    candidates for centroids to be the mean of the points within a given
+    region. These candidates are then filtered in a post-processing stage to
+    eliminate near-duplicates to form the final set of centroids.
+
+    Seeding is performed using a binning technique for scalability.
 
     Parameters
     ----------
     bandwidth : float, optional
-        Bandwidth used in the RBF kernel
-        If not set, the bandwidth is estimated.
-        See clustering.estimate_bandwidth.
+        Bandwidth used in the RBF kernel.
 
-    seeds : array [n_samples, n_features], optional
+        If not given, the bandwidth is estimated using
+        sklearn.cluster.estimate_bandwidth; see the documentation for that
+        function for hints on scalability (see also the Notes, below).
+
+    seeds : array, shape=[n_samples, n_features], optional
         Seeds used to initialize kernels. If not set,
         the seeds are calculated by clustering.get_bin_seeds
         with bandwidth as the grid size and default values for
         other parameters.
 
+    bin_seeding : boolean, optional
+        If true, initial kernel locations are not locations of all
+        points, but rather the location of the discretized version of
+        points, where points are binned onto a grid whose coarseness
+        corresponds to the bandwidth. Setting this option to True will speed
+        up the algorithm because fewer seeds will be initialized.
+        default value: False
+        Ignored if seeds argument is not None.
+
     min_bin_freq : int, optional
        To speed up the algorithm, accept only those bins with at least
        min_bin_freq points as seeds. If not defined, set to 1.
@@ -246,8 +282,8 @@ class MeanShift(BaseEstimator, ClusterMixin):
     Scalability can be boosted by using fewer seeds, for example by using
     a higher value of min_bin_freq in the get_bin_seeds function.
 
-    Note that the estimate_bandwidth function is much less scalable than
-    the mean shift algorithm and will be the bottleneck if it is used.
+    Note that the estimate_bandwidth function is much less scalable than the
+    mean shift algorithm and will be the bottleneck if it is used.
 
     References
     ----------
@@ -279,3 +315,18 @@ class MeanShift(BaseEstimator, ClusterMixin):
                        bin_seeding=self.bin_seeding,
                        cluster_all=self.cluster_all)
         return self
+
+    def predict(self, X):
+        """Predict the closest cluster each sample in X belongs to.
+
+        Parameters
+        ----------
+        X : {array-like, sparse matrix}, shape=[n_samples, n_features]
+            New data to predict.
+
+        Returns
+        -------
+        labels : array, shape [n_samples,]
+            Index of the cluster each sample belongs to.
+        """
+        return pairwise_distances_argmin(X, self.cluster_centers_)
