@@ -4,19 +4,25 @@
 # Author: Alexandre Gramfort <alexandre.gramfort@inria.fr>
 #         Olivier Grisel <olivier.grisel@ensta.org>
 #         Mathieu Blondel <mathieu@mblondel.org>
+#         Denis A. Engemann <d.engemann@fz-juelich.de>
+#
 # License: BSD 3 clause
 
+from math import log, sqrt
+import warnings
+
 import numpy as np
 from scipy import linalg
-from math import log
-import warnings
+from scipy import sparse
+from scipy.special import gammaln
 
 from ..base import BaseEstimator, TransformerMixin
 from ..utils import array2d, check_random_state, as_float_array
 from ..utils import atleast2d_or_csr
-from ..utils.extmath import fast_logdet
-from ..utils.extmath import safe_sparse_dot
-from ..utils.extmath import randomized_svd
+from ..utils import deprecated
+from ..utils.sparsefuncs import mean_variance_axis0
+from ..utils.extmath import (fast_logdet, safe_sparse_dot, randomized_svd,
+                             fast_dot)
 
 
 def _assess_dimension_(spectrum, rank, n_samples, n_features):
@@ -49,36 +55,34 @@ def _assess_dimension_(spectrum, rank, n_samples, n_features):
     if rank > len(spectrum):
         raise ValueError("The tested rank cannot exceed the rank of the"
                          " dataset")
-    from scipy.special import gammaln
 
-    pu = -rank * np.log(2)
+    pu = -rank * log(2.)
     for i in range(rank):
-        pu += (gammaln((n_features - i) / 2)
-               - np.log(np.pi) * (n_features - i) / 2)
+        pu += (gammaln((n_features - i) / 2.)
+               - log(np.pi) * (n_features - i) / 2.)
 
     pl = np.sum(np.log(spectrum[:rank]))
-    pl = -pl * n_samples / 2
+    pl = -pl * n_samples / 2.
 
     if rank == n_features:
         pv = 0
         v = 1
     else:
         v = np.sum(spectrum[rank:]) / (n_features - rank)
-        pv = -np.log(v) * n_samples * (n_features - rank) / 2
+        pv = -np.log(v) * n_samples * (n_features - rank) / 2.
 
-    m = n_features * rank - rank * (rank + 1) / 2
-    pp = np.log(2 * np.pi) * (m + rank + 1) / 2
+    m = n_features * rank - rank * (rank + 1.) / 2.
+    pp = log(2. * np.pi) * (m + rank + 1.) / 2.
 
-    pa = 0
+    pa = 0.
     spectrum_ = spectrum.copy()
     spectrum_[rank:n_features] = v
     for i in range(rank):
         for j in range(i + 1, len(spectrum)):
-            pa += (np.log((spectrum[i] - spectrum[j])
-                          * (1. / spectrum_[j] - 1. / spectrum_[i]))
-                   + np.log(n_samples))
+            pa += log((spectrum[i] - spectrum[j]) *
+                      (1. / spectrum_[j] - 1. / spectrum_[i])) + log(n_samples)
 
-    ll = pu + pl + pv + pp - pa / 2 - rank * np.log(n_samples) / 2
+    ll = pu + pl + pv + pp - pa / 2. - rank * log(n_samples) / 2.
 
     return ll
 
@@ -88,10 +92,10 @@ def _infer_dimension_(spectrum, n_samples, n_features):
 
     The dataset is described by its spectrum `spectrum`.
     """
-    ll = []
-    for rank in range(len(spectrum)):
-        ll.append(_assess_dimension_(spectrum, rank, n_samples, n_features))
-    ll = np.array(ll)
+    n_spectrum = len(spectrum)
+    ll = np.empty(n_spectrum)
+    for rank in range(n_spectrum):
+        ll[rank] = _assess_dimension_(spectrum, rank, n_samples, n_features)
     return ll.argmax()
 
 
@@ -147,11 +151,32 @@ class PCA(BaseEstimator, TransformerMixin):
         k is not set then all components are stored and the sum of explained \
         variances is equal to 1.0
 
+    `mean_` : array, [n_features]
+        Per-feature empirical mean, estimated from the training set.
+
+    `n_components_` : int
+        The estimated number of components. Relevant when n_components is set
+        to 'mle' or a number between 0 and 1 to select using explained
+        variance.
+
+    `noise_variance_` : float
+        The estimated noise covariance following the Probabilistic PCA model
+        from Tipping and Bishop 1999. See "Pattern Recognition and
+        Machine Learning" by C. Bishop, 12.2.1 p. 574 or
+        http://www.miketipping.com/papers/met-mppca.pdf. It is required to
+        computed the estimated data covariance and score samples.
+
     Notes
     -----
     For n_components='mle', this class uses the method of `Thomas P. Minka:
     Automatic Choice of Dimensionality for PCA. NIPS 2000: 598-604`
 
+    Implements the probabilistic PCA model from:
+    M. Tipping and C. Bishop, Probabilistic Principal Component Analysis,
+    Journal of the Royal Statistical Society, Series B, 61, Part 3, pp. 611-622
+    via the score and score_samples methods.
+    See http://www.miketipping.com/papers/met-mppca.pdf
+
     Due to implementation subtleties of the Singular Value Decomposition (SVD),
     which is used in this implementation, running fit twice on the same matrix
     can lead to principal components with signs flipped (change in direction).
@@ -215,19 +240,20 @@ class PCA(BaseEstimator, TransformerMixin):
 
         """
         U, S, V = self._fit(X)
-        U = U[:, :self.n_components]
+        U = U[:, :self.n_components_]
 
         if self.whiten:
             # X_new = X * V / S * sqrt(n_samples) = U * sqrt(n_samples)
-            U *= np.sqrt(X.shape[0])
+            U *= sqrt(X.shape[0])
         else:
             # X_new = X * V = U * S * V^T * V = U * S
-            U *= S[:self.n_components]
+            U *= S[:self.n_components_]
 
         return U
 
     def _fit(self, X):
-        """ Fit the model on X
+        """Fit the model on X
+
         Parameters
         ----------
         X: array-like, shape (n_samples, n_features)
@@ -247,42 +273,112 @@ class PCA(BaseEstimator, TransformerMixin):
         self.mean_ = np.mean(X, axis=0)
         X -= self.mean_
         U, S, V = linalg.svd(X, full_matrices=False)
-        self.explained_variance_ = (S ** 2) / n_samples
-        self.explained_variance_ratio_ = (self.explained_variance_ /
-                                          self.explained_variance_.sum())
+        explained_variance_ = (S ** 2) / n_samples
+        explained_variance_ratio_ = (explained_variance_ /
+                                     explained_variance_.sum())
 
         if self.whiten:
-            self.components_ = V / S[:, np.newaxis] * np.sqrt(n_samples)
+            components_ = V / (S[:, np.newaxis] / sqrt(n_samples))
         else:
-            self.components_ = V
+            components_ = V
 
-        if self.n_components == 'mle':
+        n_components = self.n_components
+        if n_components is None:
+            n_components = n_features
+        elif n_components == 'mle':
             if n_samples < n_features:
                 raise ValueError("n_components='mle' is only supported "
                                  "if n_samples >= n_features")
-            self.n_components = _infer_dimension_(self.explained_variance_,
-                                                  n_samples, n_features)
 
-        elif (self.n_components is not None
-              and 0 < self.n_components
-              and self.n_components < 1.0):
+            n_components = _infer_dimension_(explained_variance_,
+                                             n_samples, n_features)
+        elif not 0 <= n_components <= n_features:
+            raise ValueError("n_components=%r invalid for n_features=%d"
+                             % (n_components, n_features))
+
+        if 0 < n_components < 1.0:
             # number of components for which the cumulated explained variance
             # percentage is superior to the desired threshold
-            ratio_cumsum = self.explained_variance_ratio_.cumsum()
-            self.n_components = np.sum(ratio_cumsum < self.n_components) + 1
+            ratio_cumsum = explained_variance_ratio_.cumsum()
+            n_components = np.sum(ratio_cumsum < n_components) + 1
 
-        if self.n_components is not None:
-            self.components_ = self.components_[:self.n_components, :]
-            self.explained_variance_ = \
-                self.explained_variance_[:self.n_components]
-            self.explained_variance_ratio_ = \
-                self.explained_variance_ratio_[:self.n_components]
+        # Compute noise covariance using Probabilistic PCA model
+        # The sigma2 maximum likelihood (cf. eq. 12.46)
+        if n_components < n_features:
+            self.noise_variance_ = explained_variance_[n_components:].mean()
+        else:
+            self.noise_variance_ = 0.
+
+        # store n_samples to revert whitening when getting covariance
+        self.n_samples_ = n_samples
+
+        self.components_ = components_[:n_components]
+        self.explained_variance_ = explained_variance_[:n_components]
+        explained_variance_ratio_ = explained_variance_ratio_[:n_components]
+        self.explained_variance_ratio_ = explained_variance_ratio_
+        self.n_components_ = n_components
 
         return (U, S, V)
 
+    def get_covariance(self):
+        """Compute data covariance with the generative model.
+
+        ``cov = components_.T * S**2 * components_ + sigma2 * eye(n_features)``
+        where  S**2 contains the explained variances.
+
+        Returns
+        -------
+        cov : array, shape=(n_features, n_features)
+            Estimated covariance of data.
+        """
+        components_ = self.components_
+        exp_var = self.explained_variance_
+        if self.whiten:
+            components_ = components_ * np.sqrt(exp_var[:, np.newaxis])
+        exp_var_diff = np.maximum(exp_var - self.noise_variance_, 0.)
+        cov = np.dot(components_.T * exp_var_diff, components_)
+        cov.flat[::len(cov) + 1] += self.noise_variance_  # modify diag inplace
+        return cov
+
+    def get_precision(self):
+        """Compute data precision matrix with the generative model.
+
+        Equals the inverse of the covariance but computed with
+        the matrix inversion lemma for efficiency.
+
+        Returns
+        -------
+        precision : array, shape=(n_features, n_features)
+            Estimated precision of data.
+        """
+        n_features = self.components_.shape[1]
+
+        # handle corner cases first
+        if self.n_components_ == 0:
+            return np.eye(n_features) / self.noise_variance_
+        if self.n_components_ == n_features:
+            return linalg.inv(self.get_covariance())
+
+        # Get precision using matrix inversion lemma
+        components_ = self.components_
+        exp_var = self.explained_variance_
+        if self.whiten:
+            components_ = components_ * np.sqrt(exp_var[:, np.newaxis])
+        exp_var_diff = np.maximum(exp_var - self.noise_variance_, 0.)
+        precision = np.dot(components_, components_.T) / self.noise_variance_
+        precision.flat[::len(precision) + 1] += 1. / exp_var_diff
+        precision = np.dot(components_.T,
+                           np.dot(linalg.inv(precision), components_))
+        precision /= -(self.noise_variance_ ** 2)
+        precision.flat[::len(precision) + 1] += 1. / self.noise_variance_
+        return precision
+
     def transform(self, X):
         """Apply the dimensionality reduction on X.
 
+        X is projected on the first principal components previous extracted
+        from a training set.
+
         Parameters
         ----------
         X : array-like, shape (n_samples, n_features)
@@ -297,7 +393,7 @@ class PCA(BaseEstimator, TransformerMixin):
         X = array2d(X)
         if self.mean_ is not None:
             X = X - self.mean_
-        X_transformed = np.dot(X, self.components_.T)
+        X_transformed = fast_dot(X, self.components_.T)
         return X_transformed
 
     def inverse_transform(self, X):
@@ -319,9 +415,60 @@ class PCA(BaseEstimator, TransformerMixin):
         If whitening is enabled, inverse_transform does not compute the
         exact inverse operation as transform.
         """
-        return np.dot(X, self.components_) + self.mean_
+        return fast_dot(X, self.components_) + self.mean_
+
+    def score_samples(self, X):
+        """Return the log-likelihood of each sample
+
+        See. "Pattern Recognition and Machine Learning"
+        by C. Bishop, 12.2.1 p. 574
+        or http://www.miketipping.com/papers/met-mppca.pdf
+
+        Parameters
+        ----------
+        X: array, shape(n_samples, n_features)
+            The data.
+
+        Returns
+        -------
+        ll: array, shape (n_samples,)
+            Log-likelihood of each sample under the current model
+        """
+        X = array2d(X)
+        Xr = X - self.mean_
+        n_features = X.shape[1]
+        log_like = np.zeros(X.shape[0])
+        precision = self.get_precision()
+        log_like = -.5 * (Xr * (np.dot(Xr, precision))).sum(axis=1)
+        log_like -= .5 * (n_features * log(2. * np.pi)
+                          - fast_logdet(precision))
+        return log_like
+
+    def score(self, X, y=None):
+        """Return the average log-likelihood of all samples
+
+        See. "Pattern Recognition and Machine Learning"
+        by C. Bishop, 12.2.1 p. 574
+        or http://www.miketipping.com/papers/met-mppca.pdf
+
+        Parameters
+        ----------
+        X: array, shape(n_samples, n_features)
+            The data.
+
+        Returns
+        -------
+        ll: float
+            Average log-likelihood of the samples under the current model
+        """
+        return np.mean(self.score_samples(X))
 
 
+@deprecated("ProbabilisticPCA will be removed in 0.16. WARNING: the "
+            "covariance estimation was previously incorrect, your "
+            "output might be different than under the previous versions. "
+            "Use PCA that implements score and score_samples. To work with "
+            "homoscedastic=False, you should use FactorAnalysis.")
 class ProbabilisticPCA(PCA):
     """Additional layer on top of PCA that adds a probabilistic evaluation"""
     __doc__ += PCA.__doc__
@@ -338,25 +485,33 @@ class ProbabilisticPCA(PCA):
             If True, average variance across remaining dimensions
         """
         PCA.fit(self, X)
-        n_features = X.shape[1]
-        self._dim = n_features
-        Xr = X - self.mean_
-        Xr -= np.dot(np.dot(Xr, self.components_.T), self.components_)
-        n_samples = X.shape[0]
-        if self.n_components is None or n_features <= self.n_components:
-            delta = np.zeros(n_features)
-        elif homoscedastic:
-            delta = ((Xr ** 2).sum() * np.ones(n_features)
-                     / (n_samples * n_features))
-        else:
-            delta = (Xr ** 2).mean(0) / (n_features - self.n_components)
-        self.covariance_ = np.diag(delta)
+
+        n_samples, n_features = X.shape
         n_components = self.n_components
         if n_components is None:
             n_components = n_features
-        for k in range(n_components):
-            add_cov = np.outer(self.components_[k], self.components_[k])
-            self.covariance_ += self.explained_variance_[k] * add_cov
+
+        explained_variance = self.explained_variance_.copy()
+        if homoscedastic:
+            explained_variance -= self.noise_variance_
+
+        # Make the low rank part of the estimated covariance
+        self.covariance_ = np.dot(self.components_[:n_components].T *
+                                  explained_variance,
+                                  self.components_[:n_components])
+
+        if n_features == n_components:
+            delta = 0.
+        elif homoscedastic:
+            delta = self.noise_variance_
+        else:
+            Xr = X - self.mean_
+            Xr -= np.dot(np.dot(Xr, self.components_.T), self.components_)
+            delta = (Xr ** 2).mean(axis=0) / (n_features - n_components)
+
+        # Add delta to the diagonal without extra allocation
+        self.covariance_.flat[::n_features + 1] += delta
+
         return self
 
     def score(self, X, y=None):
@@ -378,7 +533,7 @@ class ProbabilisticPCA(PCA):
         self.precision_ = linalg.inv(self.covariance_)
         log_like = -.5 * (Xr * (np.dot(Xr, self.precision_))).sum(axis=1)
         log_like -= .5 * (fast_logdet(self.covariance_)
-                          + n_features * log(2 * np.pi))
+                          + n_features * log(2. * np.pi))
         return log_like
 
 
@@ -427,6 +582,9 @@ class RandomizedPCA(BaseEstimator, TransformerMixin):
         k is not set then all components are stored and the sum of explained \
         variances is equal to 1.0
 
+    `mean_` : array, [n_features]
+        Per-feature empirical mean, estimated from the training set.
+
     Examples
     --------
     >>> import numpy as np
@@ -470,11 +628,10 @@ class RandomizedPCA(BaseEstimator, TransformerMixin):
         self.copy = copy
         self.iterated_power = iterated_power
         self.whiten = whiten
-        self.mean_ = None
         self.random_state = random_state
 
     def fit(self, X, y=None):
-        """Fit the model with X.
+        """Fit the model with X by extracting the first principal components.
 
         Parameters
         ----------
@@ -505,7 +662,7 @@ class RandomizedPCA(BaseEstimator, TransformerMixin):
             The input data, copied, centered and whitened when requested.
         """
         random_state = check_random_state(self.random_state)
-        if hasattr(X, 'todense'):
+        if sparse.issparse(X):
             warnings.warn("Sparse matrix support is deprecated"
                           " and will be dropped in 0.16."
                           " Use TruncatedSVD instead.",
@@ -516,7 +673,9 @@ class RandomizedPCA(BaseEstimator, TransformerMixin):
 
         n_samples = X.shape[0]
 
-        if not hasattr(X, 'todense'):
+        if sparse.issparse(X):
+            self.mean_ = None
+        else:
             # Center data
             self.mean_ = np.mean(X, axis=0)
             X -= self.mean_
@@ -530,10 +689,15 @@ class RandomizedPCA(BaseEstimator, TransformerMixin):
                                  random_state=random_state)
 
         self.explained_variance_ = exp_var = (S ** 2) / n_samples
-        self.explained_variance_ratio_ = exp_var / exp_var.sum()
+        if sparse.issparse(X):
+            _, full_var = mean_variance_axis0(X)
+            full_var = full_var.sum()
+        else:
+            full_var = np.var(X, axis=0).sum()
+        self.explained_variance_ratio_ = exp_var / full_var
 
         if self.whiten:
-            self.components_ = V / S[:, np.newaxis] * np.sqrt(n_samples)
+            self.components_ = V / S[:, np.newaxis] * sqrt(n_samples)
         else:
             self.components_ = V
 
@@ -542,6 +706,9 @@ class RandomizedPCA(BaseEstimator, TransformerMixin):
     def transform(self, X, y=None):
         """Apply dimensionality reduction on X.
 
+        X is projected on the first principal components previous extracted
+        from a training set.
+
         Parameters
         ----------
         X : array-like, shape (n_samples, n_features)
@@ -562,7 +729,7 @@ class RandomizedPCA(BaseEstimator, TransformerMixin):
         return X
 
     def fit_transform(self, X, y=None):
-        """Apply dimensionality reduction on X.
+        """Fit the model with X and apply the dimensionality reduction on X.
 
         Parameters
         ----------
