@@ -8,6 +8,7 @@ randomized trees. Single and multi-output problems are both handled.
 #          Brian Holt <bdholt1@gmail.com>
 #          Noel Dawe <noel@dawe.me>
 #          Satrajit Gosh <satrajit.ghosh@gmail.com>
+#          Joly Arnaud <arnaud.v.joly@gmail.com>
 # Licence: BSD 3 clause
 
 from __future__ import division
@@ -22,13 +23,14 @@ from ..externals import six
 from ..externals.six.moves import xrange
 from ..feature_selection.from_model import _LearntSelectorMixin
 from ..utils import array2d, check_random_state
-from ..utils.fixes import unique
 from ..utils.validation import check_arrays
 
-from ._tree import Criterion, Splitter, Tree
+from ._tree import Criterion
+from ._tree import Splitter
+from ._tree import DepthFirstTreeBuilder, BestFirstTreeBuilder
+from ._tree import Tree
 from . import _tree
 
-
 __all__ = ["DecisionTreeClassifier",
            "DecisionTreeRegressor",
            "ExtraTreeClassifier",
@@ -43,8 +45,10 @@ DTYPE = _tree.DTYPE
 DOUBLE = _tree.DOUBLE
 
 CRITERIA_CLF = {"gini": _tree.Gini, "entropy": _tree.Entropy}
-CRITERIA_REG = {"mse": _tree.MSE}
-SPLITTERS = {"best": _tree.BestSplitter, "random": _tree.RandomSplitter}
+CRITERIA_REG = {"mse": _tree.MSE, "friedman_mse": _tree.FriedmanMSE}
+SPLITTERS = {"best": _tree.BestSplitter,
+             "presort-best": _tree.PresortBestSplitter,
+             "random": _tree.RandomSplitter}
 
 
 # =============================================================================
@@ -67,6 +71,7 @@ class BaseDecisionTree(six.with_metaclass(ABCMeta, BaseEstimator,
                  min_samples_split,
                  min_samples_leaf,
                  max_features,
+                 max_leaf_nodes,
                  random_state):
         self.criterion = criterion
         self.splitter = splitter
@@ -75,14 +80,15 @@ class BaseDecisionTree(six.with_metaclass(ABCMeta, BaseEstimator,
         self.min_samples_leaf = min_samples_leaf
         self.max_features = max_features
         self.random_state = random_state
+        self.max_leaf_nodes = max_leaf_nodes
 
         self.n_features_ = None
         self.n_outputs_ = None
         self.classes_ = None
         self.n_classes_ = None
 
-        self.splitter_ = None
         self.tree_ = None
+        self.max_features_ = None
 
     def fit(self, X, y, sample_mask=None, X_argsorted=None, check_input=True,
             sample_weight=None):
@@ -95,10 +101,9 @@ class BaseDecisionTree(six.with_metaclass(ABCMeta, BaseEstimator,
             efficiency.
 
         y : array-like, shape = [n_samples] or [n_samples, n_outputs]
-            The target values (integers that correspond to classes in
-            classification, real numbers in regression).
-            Use ``dtype=np.float64`` and ``order='C'`` for maximum
-            efficiency.
+            The target values (class labels in classification, real numbers in
+            regression). In the regression case, use ``dtype=np.float64`` and
+            ``order='C'`` for maximum efficiency.
 
         sample_weight : array-like, shape = [n_samples] or None
             Sample weights. If None, then samples are equally weighted. Splits
@@ -129,8 +134,7 @@ class BaseDecisionTree(six.with_metaclass(ABCMeta, BaseEstimator,
 
         # Convert data
         if check_input:
-            X, = check_arrays(X, dtype=DTYPE, sparse_format="dense",
-                              check_ccontiguous=True)
+            X, = check_arrays(X, dtype=DTYPE, sparse_format="dense")
 
         # Determine output settings
         n_samples, self.n_features_ = X.shape
@@ -152,7 +156,7 @@ class BaseDecisionTree(six.with_metaclass(ABCMeta, BaseEstimator,
             self.n_classes_ = []
 
             for k in xrange(self.n_outputs_):
-                classes_k, y[:, k] = unique(y[:, k], return_inverse=True)
+                classes_k, y[:, k] = np.unique(y[:, k], return_inverse=True)
                 self.classes_.append(classes_k)
                 self.n_classes_.append(classes_k.shape[0])
 
@@ -166,7 +170,10 @@ class BaseDecisionTree(six.with_metaclass(ABCMeta, BaseEstimator,
             y = np.ascontiguousarray(y, dtype=DOUBLE)
 
         # Check parameters
-        max_depth = (2 ** 31) - 1 if self.max_depth is None else self.max_depth
+        max_depth = ((2 ** 31) - 1 if self.max_depth is None
+                     else self.max_depth)
+        max_leaf_nodes = (-1 if self.max_leaf_nodes is None
+                          else self.max_leaf_nodes)
 
         if isinstance(self.max_features, six.string_types):
             if self.max_features == "auto":
@@ -187,7 +194,12 @@ class BaseDecisionTree(six.with_metaclass(ABCMeta, BaseEstimator,
         elif isinstance(self.max_features, (numbers.Integral, np.integer)):
             max_features = self.max_features
         else:  # float
-            max_features = int(self.max_features * self.n_features_)
+            if self.max_features > 0.0:
+                max_features = max(1, int(self.max_features * self.n_features_))
+            else:
+                max_features = 0
+
+        self.max_features_ = max_features
 
         if len(y) != n_samples:
             raise ValueError("Number of labels=%d does not match "
@@ -200,6 +212,12 @@ class BaseDecisionTree(six.with_metaclass(ABCMeta, BaseEstimator,
             raise ValueError("max_depth must be greater than zero. ")
         if not (0 < max_features <= self.n_features_):
             raise ValueError("max_features must be in (0, n_features]")
+        if not isinstance(max_leaf_nodes, (numbers.Integral, np.integer)):
+            raise ValueError("max_leaf_nodes must be integral number but was "
+                             "%r" % max_leaf_nodes)
+        if -1 < max_leaf_nodes < 2:
+            raise ValueError(("max_leaf_nodes {0} must be either smaller than "
+                              "0 or larger than 1").format(max_leaf_nodes))
 
         if sample_weight is not None:
             if (getattr(sample_weight, "dtype", None) != DOUBLE or
@@ -231,18 +249,22 @@ class BaseDecisionTree(six.with_metaclass(ABCMeta, BaseEstimator,
         splitter = self.splitter
         if not isinstance(self.splitter, Splitter):
             splitter = SPLITTERS[self.splitter](criterion,
-                                                max_features,
+                                                self.max_features_,
                                                 self.min_samples_leaf,
                                                 random_state)
 
-        self.criterion_ = criterion
-        self.splitter_ = splitter
-        self.tree_ = Tree(self.n_features_, self.n_classes_,
-                          self.n_outputs_, splitter, max_depth,
-                          min_samples_split, self.min_samples_leaf,
-                          random_state)
+        self.tree_ = Tree(self.n_features_, self.n_classes_, self.n_outputs_)
 
-        self.tree_.build(X, y, sample_weight=sample_weight)
+        # Use BestFirst if max_leaf_nodes given; use DepthFirst otherwise
+        if max_leaf_nodes < 0:
+            builder = DepthFirstTreeBuilder(splitter, min_samples_split,
+                                            self.min_samples_leaf, max_depth)
+        else:
+            builder = BestFirstTreeBuilder(splitter, min_samples_split,
+                                           self.min_samples_leaf, max_depth,
+                                           max_leaf_nodes)
+
+        builder.build(self.tree_, X, y, sample_weight)
 
         if self.n_outputs_ == 1:
             self.n_classes_ = self.n_classes_[0]
@@ -338,6 +360,11 @@ class DecisionTreeClassifier(BaseDecisionTree, ClassifierMixin):
         The function to measure the quality of a split. Supported criteria are
         "gini" for the Gini impurity and "entropy" for the information gain.
 
+    splitter : string, optional (default="best")
+        The strategy used to choose the split at each node. Supported
+        strategies are "best" to choose the best split and "random" to choose
+        the best random split.
+
     max_features : int, float, string or None, optional (default=None)
         The number of features to consider when looking for the best split:
           - If int, then consider `max_features` features at each split.
@@ -349,17 +376,28 @@ class DecisionTreeClassifier(BaseDecisionTree, ClassifierMixin):
           - If "log2", then `max_features=log2(n_features)`.
           - If None, then `max_features=n_features`.
 
-    max_depth : integer or None, optional (default=None)
+        Note: the search for a split does not stop until at least one
+        valid partition of the node samples is found, even if it requires to
+        effectively inspect more than ``max_features`` features.
+
+    max_depth : int or None, optional (default=None)
         The maximum depth of the tree. If None, then nodes are expanded until
         all leaves are pure or until all leaves contain less than
         min_samples_split samples.
+        Ignored if ``max_samples_leaf`` is not None.
 
-    min_samples_split : integer, optional (default=2)
+    min_samples_split : int, optional (default=2)
         The minimum number of samples required to split an internal node.
 
-    min_samples_leaf : integer, optional (default=1)
+    min_samples_leaf : int, optional (default=1)
         The minimum number of samples required to be at a leaf node.
 
+    max_leaf_nodes : int or None, optional (default=None)
+        Grow a tree with ``max_leaf_nodes`` in best-first fashion.
+        Best nodes are defined as relative reduction in impurity.
+        If None then unlimited number of leaf nodes.
+        If not None then ``max_depth`` will be ignored.
+
     random_state : int, RandomState instance or None, optional (default=None)
         If int, random_state is the seed used by the random number generator;
         If RandomState instance, random_state is the random number generator;
@@ -371,6 +409,9 @@ class DecisionTreeClassifier(BaseDecisionTree, ClassifierMixin):
     `tree_` : Tree object
         The underlying Tree object.
 
+    `max_features_` : int,
+        The infered value of max_features.
+
     `classes_` : array of shape = [n_classes] or a list of such arrays
         The classes labels (single output problem),
         or a list of arrays of class labels (multi-output problem).
@@ -426,14 +467,18 @@ class DecisionTreeClassifier(BaseDecisionTree, ClassifierMixin):
                  max_features=None,
                  random_state=None,
                  min_density=None,
-                 compute_importances=None):
-        super(DecisionTreeClassifier, self).__init__(criterion,
-                                                     splitter,
-                                                     max_depth,
-                                                     min_samples_split,
-                                                     min_samples_leaf,
-                                                     max_features,
-                                                     random_state)
+                 compute_importances=None,
+                 max_leaf_nodes=None):
+        super(DecisionTreeClassifier, self).__init__(
+            criterion=criterion,
+            splitter=splitter,
+            max_depth=max_depth,
+            min_samples_split=min_samples_split,
+            min_samples_leaf=min_samples_leaf,
+            max_features=max_features,
+            max_leaf_nodes=max_leaf_nodes,
+            random_state=random_state)
+
         if min_density is not None:
             warn("The min_density parameter is deprecated as of version 0.14 "
                  "and will be removed in 0.16.", DeprecationWarning)
@@ -457,8 +502,8 @@ class DecisionTreeClassifier(BaseDecisionTree, ClassifierMixin):
         -------
         p : array of shape = [n_samples, n_classes], or a list of n_outputs
             such arrays if n_outputs > 1.
-            The class probabilities of the input samples. Classes are ordered
-            by arithmetical order.
+            The class probabilities of the input samples. The order of the
+            classes corresponds to that in the attribute `classes_`.
         """
         if getattr(X, "dtype", None) != DTYPE or X.ndim != 2:
             X = array2d(X, dtype=DTYPE)
@@ -508,8 +553,8 @@ class DecisionTreeClassifier(BaseDecisionTree, ClassifierMixin):
         -------
         p : array of shape = [n_samples, n_classes], or a list of n_outputs
             such arrays if n_outputs > 1.
-            The class log-probabilities of the input samples. Classes are
-            ordered by arithmetical order.
+            The class log-probabilities of the input samples. The order of the
+            classes corresponds to that in the attribute `classes_`.
         """
         proba = self.predict_proba(X)
 
@@ -524,7 +569,7 @@ class DecisionTreeClassifier(BaseDecisionTree, ClassifierMixin):
 
 
 class DecisionTreeRegressor(BaseDecisionTree, RegressorMixin):
-    """A tree regressor.
+    """A decision tree regressor.
 
     Parameters
     ----------
@@ -532,6 +577,11 @@ class DecisionTreeRegressor(BaseDecisionTree, RegressorMixin):
         The function to measure the quality of a split. The only supported
         criterion is "mse" for the mean squared error.
 
+    splitter : string, optional (default="best")
+        The strategy used to choose the split at each node. Supported
+        strategies are "best" to choose the best split and "random" to choose
+        the best random split.
+
     max_features : int, float, string or None, optional (default=None)
         The number of features to consider when looking for the best split:
           - If int, then consider `max_features` features at each split.
@@ -543,17 +593,28 @@ class DecisionTreeRegressor(BaseDecisionTree, RegressorMixin):
           - If "log2", then `max_features=log2(n_features)`.
           - If None, then `max_features=n_features`.
 
-    max_depth : integer or None, optional (default=None)
+        Note: the search for a split does not stop until at least one
+        valid partition of the node samples is found, even if it requires to
+        effectively inspect more than ``max_features`` features.
+
+    max_depth : int or None, optional (default=None)
         The maximum depth of the tree. If None, then nodes are expanded until
         all leaves are pure or until all leaves contain less than
         min_samples_split samples.
+        Ignored if ``max_samples_leaf`` is not None.
 
-    min_samples_split : integer, optional (default=2)
+    min_samples_split : int, optional (default=2)
         The minimum number of samples required to split an internal node.
 
-    min_samples_leaf : integer, optional (default=1)
+    min_samples_leaf : int, optional (default=1)
         The minimum number of samples required to be at a leaf node.
 
+    max_leaf_nodes : int or None, optional (default=None)
+        Grow a tree with ``max_leaf_nodes`` in best-first fashion.
+        Best nodes are defined as relative reduction in impurity.
+        If None then unlimited number of leaf nodes.
+        If not None then ``max_depth`` will be ignored.
+
     random_state : int, RandomState instance or None, optional (default=None)
         If int, random_state is the seed used by the random number generator;
         If RandomState instance, random_state is the random number generator;
@@ -565,6 +626,9 @@ class DecisionTreeRegressor(BaseDecisionTree, RegressorMixin):
     `tree_` : Tree object
         The underlying Tree object.
 
+    `max_features_` : int,
+        The infered value of max_features.
+
     `feature_importances_` : array of shape = [n_features]
         The feature importances.
         The higher, the more important the feature.
@@ -612,14 +676,18 @@ class DecisionTreeRegressor(BaseDecisionTree, RegressorMixin):
                  max_features=None,
                  random_state=None,
                  min_density=None,
-                 compute_importances=None):
-        super(DecisionTreeRegressor, self).__init__(criterion,
-                                                    splitter,
-                                                    max_depth,
-                                                    min_samples_split,
-                                                    min_samples_leaf,
-                                                    max_features,
-                                                    random_state)
+                 compute_importances=None,
+                 max_leaf_nodes=None):
+        super(DecisionTreeRegressor, self).__init__(
+            criterion=criterion,
+            splitter=splitter,
+            max_depth=max_depth,
+            min_samples_split=min_samples_split,
+            min_samples_leaf=min_samples_leaf,
+            max_features=max_features,
+            max_leaf_nodes=max_leaf_nodes,
+            random_state=random_state)
+
         if min_density is not None:
             warn("The min_density parameter is deprecated as of version 0.14 "
                  "and will be removed in 0.16.", DeprecationWarning)
@@ -663,14 +731,18 @@ class ExtraTreeClassifier(DecisionTreeClassifier):
                  max_features="auto",
                  random_state=None,
                  min_density=None,
-                 compute_importances=None):
-        super(ExtraTreeClassifier, self).__init__(criterion,
-                                                  splitter,
-                                                  max_depth,
-                                                  min_samples_split,
-                                                  min_samples_leaf,
-                                                  max_features,
-                                                  random_state)
+                 compute_importances=None,
+                 max_leaf_nodes=None):
+        super(ExtraTreeClassifier, self).__init__(
+            criterion=criterion,
+            splitter=splitter,
+            max_depth=max_depth,
+            min_samples_split=min_samples_split,
+            min_samples_leaf=min_samples_leaf,
+            max_features=max_features,
+            max_leaf_nodes=max_leaf_nodes,
+            random_state=random_state)
+
         if min_density is not None:
             warn("The min_density parameter is deprecated as of version 0.14 "
                  "and will be removed in 0.16.", DeprecationWarning)
@@ -714,14 +786,18 @@ class ExtraTreeRegressor(DecisionTreeRegressor):
                  max_features="auto",
                  random_state=None,
                  min_density=None,
-                 compute_importances=None):
-        super(ExtraTreeRegressor, self).__init__(criterion,
-                                                 splitter,
-                                                 max_depth,
-                                                 min_samples_split,
-                                                 min_samples_leaf,
-                                                 max_features,
-                                                 random_state)
+                 compute_importances=None,
+                 max_leaf_nodes=None):
+        super(ExtraTreeRegressor, self).__init__(
+            criterion=criterion,
+            splitter=splitter,
+            max_depth=max_depth,
+            min_samples_split=min_samples_split,
+            min_samples_leaf=min_samples_leaf,
+            max_features=max_features,
+            max_leaf_nodes=max_leaf_nodes,
+            random_state=random_state)
+
         if min_density is not None:
             warn("The min_density parameter is deprecated as of version 0.14 "
                  "and will be removed in 0.16.", DeprecationWarning)
