@@ -1,7 +1,6 @@
 """
 The :mod:`sklearn.utils` module includes various utilities.
 """
-
 from collections import Sequence
 
 import numpy as np
@@ -20,10 +19,7 @@ from sklearn.utils.sparsetools import minimum_spanning_tree
 __all__ = ["murmurhash3_32", "as_float_array", "check_arrays", "safe_asarray",
            "assert_all_finite", "array2d", "atleast2d_or_csc",
            "atleast2d_or_csr", "warn_if_not_float", "check_random_state",
-           "compute_class_weight",  "minimum_spanning_tree", "column_or_1d"]
-
-# Make sure that DeprecationWarning get printed
-warnings.simplefilter("always", DeprecationWarning)
+           "compute_class_weight", "minimum_spanning_tree", "column_or_1d"]
 
 
 class deprecated(object):
@@ -324,9 +320,37 @@ def safe_sqr(X, copy=True):
     return X
 
 
-def gen_even_slices(n, n_packs):
+def gen_batches(n, batch_size):
+    """Generator to create slices containing batch_size elements, from 0 to n.
+
+    The last slice may contain less than batch_size elements, when batch_size
+    does not divide n.
+
+    Examples
+    --------
+    >>> from sklearn.utils import gen_batches
+    >>> list(gen_batches(7, 3))
+    [slice(0, 3, None), slice(3, 6, None), slice(6, 7, None)]
+    >>> list(gen_batches(6, 3))
+    [slice(0, 3, None), slice(3, 6, None)]
+    >>> list(gen_batches(2, 3))
+    [slice(0, 2, None)]
+    """
+    start = 0
+    for _ in range(int(n // batch_size)):
+        end = start + batch_size
+        yield slice(start, end)
+        start = end
+    if start < n:
+        yield slice(start, n)
+
+
+def gen_even_slices(n, n_packs, n_samples=None):
     """Generator to create n_packs slices going up to n.
 
+    Pass n_samples when the slices are to be used for sparse matrix indexing;
+    slicing off-the-end raises an exception, while it works for NumPy arrays.
+
     Examples
     --------
     >>> from sklearn.utils import gen_even_slices
@@ -346,6 +370,8 @@ def gen_even_slices(n, n_packs):
             this_n += 1
         if this_n > 0:
             end = start + this_n
+            if n_samples is not None:
+                end = min(n_samples, end)
             yield slice(start, end, None)
             start = end
 
