@@ -6,12 +6,18 @@
 #          Mathieu Blondel
 #          Olivier Grisel
 #          Arnaud Joly
+#          Denis Engemann
 # License: BSD 3 clause
 import inspect
 import pkgutil
 import warnings
+import sys
+import re
+import platform
+import os
 
 import scipy as sp
+import scipy.io
 from functools import wraps
 try:
     # Python 2
@@ -24,7 +30,6 @@ except ImportError:
 
 import sklearn
 from sklearn.base import BaseEstimator
-from .fixes import savemat
 
 # Conveniently import all assertions in one place.
 from nose.tools import assert_equal
@@ -45,10 +50,10 @@ import numpy as np
 from sklearn.base import (ClassifierMixin, RegressorMixin, TransformerMixin,
                           ClusterMixin)
 
-__all__ = ["assert_equal", "assert_not_equal", "assert_raises", "raises",
-           "with_setup", "assert_true", "assert_false", "assert_almost_equal",
-           "assert_array_equal", "assert_array_almost_equal",
-           "assert_array_less"]
+__all__ = ["assert_equal", "assert_not_equal", "assert_raises",
+           "assert_raises_regexp", "raises", "with_setup", "assert_true",
+           "assert_false", "assert_almost_equal", "assert_array_equal",
+           "assert_array_almost_equal", "assert_array_less"]
 
 
 try:
@@ -62,6 +67,28 @@ except ImportError:
     def assert_not_in(x, container):
         assert_false(x in container, msg="%r in %r" % (x, container))
 
+try:
+    from nose.tools import assert_raises_regexp
+except ImportError:
+    # for Py 2.6
+    def assert_raises_regexp(expected_exception, expected_regexp,
+                             callable_obj=None, *args, **kwargs):
+        """Helper function to check for message patterns in exceptions"""
+
+        not_raised = False
+        try:
+            callable_obj(*args, **kwargs)
+            not_raised = True
+        except Exception as e:
+            error_message = str(e)
+            if not re.compile(expected_regexp).match(error_message):
+                raise AssertionError("Error message should match pattern "
+                                     "'%s'. '%s' does not." %
+                                     (expected_regexp, error_message))
+        if not_raised:
+            raise AssertionError("Should have raised %r" %
+                                 expected_exception(expected_regexp))
+
 
 def _assert_less(a, b, msg=None):
     message = "%r is not lower than %r" % (a, b)
@@ -79,13 +106,34 @@ def _assert_greater(a, b, msg=None):
 
 # To remove when we support numpy 1.7
 def assert_warns(warning_class, func, *args, **kw):
+    """Test that a certain warning occurs.
+
+    Parameters
+    ----------
+    warning_class : the warning class
+        The class to test for, e.g. UserWarning.
+
+    func : callable
+        Calable object to trigger warnings.
+
+    *args : the positional arguments to `func`.
+
+    **kw : the keyword arguments to `func`
+
+    Returns
+    -------
+
+    result : the return value of `func`
+
+    """
+
+    # very important to avoid uncontrolled state propagation
+    clean_warning_registry()
     with warnings.catch_warnings(record=True) as w:
         # Cause all warnings to always be triggered.
         warnings.simplefilter("always")
-
         # Trigger a warning.
         result = func(*args, **kw)
-
         # Verify some things
         if not len(w) > 0:
             raise AssertionError("No warning raised when calling %s"
@@ -99,10 +147,71 @@ def assert_warns(warning_class, func, *args, **kw):
     return result
 
 
+def assert_warns_message(warning_class, message, func, *args, **kw):
+    # very important to avoid uncontrolled state propagation
+    """Test that a certain warning occurs and with a certain message.
+
+    Parameters
+    ----------
+    warning_class : the warning class
+        The class to test for, e.g. UserWarning.
+
+    message : str | callable
+        The entire message or a substring to  test for. If callable,
+        it takes a string as argument and will trigger an assertion error
+        if it returns `False`.
+
+    func : callable
+        Calable object to trigger warnings.
+
+    *args : the positional arguments to `func`.
+
+    **kw : the keyword arguments to `func`.
+
+    Returns
+    -------
+
+    result : the return value of `func`
+
+    """
+    clean_warning_registry()
+    with warnings.catch_warnings(record=True) as w:
+        # Cause all warnings to always be triggered.
+        warnings.simplefilter("always")
+        # Trigger a warning.
+        result = func(*args, **kw)
+        # Verify some things
+        if not len(w) > 0:
+            raise AssertionError("No warning raised when calling %s"
+                                 % func.__name__)
+
+        if not w[0].category is warning_class:
+            raise AssertionError("First warning for %s is not a "
+                                 "%s( is %s)"
+                                 % (func.__name__, warning_class, w[0]))
+
+        # substring will match, the entire message with typo won't
+        msg = w[0].message  # For Python 3 compatibility
+        msg = str(msg.args[0] if hasattr(msg, 'args') else msg)
+        if callable(message):  # add support for certain tests
+            check_in_message = message
+        else:
+            check_in_message = lambda msg: message in msg
+        if not check_in_message(msg):
+            raise AssertionError("The message received ('%s') for <%s> is "
+                                 "not the one you expected ('%s')"
+                                 % (msg, func.__name__,  message
+                                    ))
+    return result
+
+
 # To remove when we support numpy 1.7
 def assert_no_warnings(func, *args, **kw):
     # XXX: once we may depend on python >= 2.6, this can be replaced by the
+
     # warnings module context manager.
+    # very important to avoid uncontrolled state propagation
+    clean_warning_registry()
     with warnings.catch_warnings(record=True) as w:
         warnings.simplefilter('always')
 
@@ -112,6 +221,104 @@ def assert_no_warnings(func, *args, **kw):
                                  % (func.__name__, w))
     return result
 
+
+def ignore_warnings(obj=None):
+    """ Context manager and decorator to ignore warnings
+
+    Note. Using this (in both variants) will clear all warnings
+    from all python modules loaded. In case you need to test
+    cross-module-warning-logging this is not your tool of choice.
+
+    Examples
+    --------
+    >>> with ignore_warnings():
+    ...     warnings.warn('buhuhuhu')
+
+    >>> def nasty_warn():
+    ...    warnings.warn('buhuhuhu')
+    ...    print(42)
+
+    >>> ignore_warnings(nasty_warn)()
+    42
+
+    """
+    if callable(obj):
+        return _ignore_warnings(obj)
+    else:
+        return _IgnoreWarnings()
+
+
+def _ignore_warnings(fn):
+    """Decorator to catch and hide warnings without visual nesting"""
+    @wraps(fn)
+    def wrapper(*args, **kwargs):
+        # very important to avoid uncontrolled state propagation
+        clean_warning_registry()
+        with warnings.catch_warnings(record=True) as w:
+            warnings.simplefilter('always')
+            return fn(*args, **kwargs)
+            w[:] = []
+
+    return wrapper
+
+
+class _IgnoreWarnings(object):
+
+    """Improved and simplified Python warnings context manager
+
+    Copied from Python 2.7.5 and modified as required.
+    """
+
+    def __init__(self):
+        """
+        Parameters
+        ==========
+        category : warning class
+            The category to filter. Defaults to Warning. If None,
+            all categories will be muted.
+        """
+        self._record = True
+        self._module = sys.modules['warnings']
+        self._entered = False
+        self.log = []
+
+    def __repr__(self):
+        args = []
+        if self._record:
+            args.append("record=True")
+        if self._module is not sys.modules['warnings']:
+            args.append("module=%r" % self._module)
+        name = type(self).__name__
+        return "%s(%s)" % (name, ", ".join(args))
+
+    def __enter__(self):
+        clean_warning_registry()  # be safe and not propagate state + chaos
+        warnings.simplefilter('always')
+        if self._entered:
+            raise RuntimeError("Cannot enter %r twice" % self)
+        self._entered = True
+        self._filters = self._module.filters
+        self._module.filters = self._filters[:]
+        self._showwarning = self._module.showwarning
+        if self._record:
+            self.log = []
+
+            def showwarning(*args, **kwargs):
+                self.log.append(warnings.WarningMessage(*args, **kwargs))
+            self._module.showwarning = showwarning
+            return self.log
+        else:
+            return None
+
+    def __exit__(self, *exc_info):
+        if not self._entered:
+            raise RuntimeError("Cannot exit %r without entering first" % self)
+        self._module.filters = self._filters
+        self._module.showwarning = self._showwarning
+        self.log[:] = []
+        clean_warning_registry()  # be safe and not propagate state + chaos
+
+
 try:
     from nose.tools import assert_less
 except ImportError:
@@ -128,7 +335,7 @@ def _assert_allclose(actual, desired, rtol=1e-7, atol=0,
     actual, desired = np.asanyarray(actual), np.asanyarray(desired)
     if np.allclose(actual, desired, rtol=rtol, atol=atol):
         return
-    msg = ('Array not equal to tolerance rtol=%g, atol=%g:'
+    msg = ('Array not equal to tolerance rtol=%g, atol=%g: '
            'actual %s, desired %s') % (rtol, atol, actual, desired)
     raise AssertionError(msg)
 
@@ -179,7 +386,7 @@ def fake_mldata(columns_dict, dataname, matfile, ordering=None):
     for i, name in enumerate(ordering):
         datasets['mldata_descr_ordering'][0, i] = name
 
-    savemat(matfile, datasets, oned_as='column')
+    scipy.io.savemat(matfile, datasets, oned_as='column')
 
 
 class mock_mldata_urlopen(object):
@@ -285,9 +492,9 @@ def all_estimators(include_meta_estimators=False, include_other=False,
     path = sklearn.__path__
     for importer, modname, ispkg in pkgutil.walk_packages(
             path=path, prefix='sklearn.', onerror=lambda x: None):
-        module = __import__(modname, fromlist="dummy")
         if ".tests." in modname:
             continue
+        module = __import__(modname, fromlist="dummy")
         classes = inspect.getmembers(module, inspect.isclass)
         all_classes.extend(classes)
 
@@ -346,3 +553,42 @@ def if_matplotlib(func):
         else:
             return func(*args, **kwargs)
     return run_test
+
+
+def if_not_mac_os(versions=('10.7', '10.8', '10.9'),
+                  message='Multi-process bug in Mac OS X >= 10.7 '
+                          '(see issue #636)'):
+    """Test decorator that skips test if OS is Mac OS X and its
+    major version is one of ``versions``.
+    """
+    mac_version, _, _ = platform.mac_ver()
+    skip = '.'.join(mac_version.split('.')[:2]) in versions
+    def decorator(func):
+        if skip:
+            @wraps(func)
+            def func(*args, **kwargs):
+                raise SkipTest(message)
+        return func
+    return decorator
+
+
+def clean_warning_registry():
+    """Safe way to reset warnings """
+    warnings.resetwarnings()
+    reg = "__warningregistry__"
+    for mod in sys.modules.copy().values():
+        if hasattr(mod, reg):
+            getattr(mod, reg).clear()
+
+def check_skip_network():
+    if int(os.environ.get('SKLEARN_SKIP_NETWORK_TESTS', 0)):
+        raise SkipTest("Text tutorial requires large dataset download")
+
+
+def check_skip_travis():
+    """Skip test if being run on Travis."""
+    if os.environ.get('TRAVIS') == "true":
+        raise SkipTest("This test needs to be skipped on Travis")
+
+with_network = with_setup(check_skip_network)
+with_travis = with_setup(check_skip_travis)
