@@ -1,13 +1,23 @@
 import numpy as np
 from itertools import product
+from functools import partial
 from sklearn.externals.six.moves import xrange
 from sklearn.externals.six import iteritems
 
+from scipy.sparse import issparse
+from scipy.sparse import csc_matrix
+from scipy.sparse import csr_matrix
+from scipy.sparse import coo_matrix
+from scipy.sparse import dok_matrix
+from scipy.sparse import lil_matrix
+
 from sklearn.utils.testing import assert_array_equal
 from sklearn.utils.testing import assert_equal
 from sklearn.utils.testing import assert_true
 from sklearn.utils.testing import assert_false
 from sklearn.utils.testing import assert_raises
+from sklearn.utils.testing import assert_warns
+from sklearn.utils.testing import ignore_warnings
 
 from sklearn.utils.multiclass import unique_labels
 from sklearn.utils.multiclass import is_label_indicator_matrix
@@ -17,17 +27,20 @@ from sklearn.utils.multiclass import type_of_target
 
 EXAMPLES = {
     'multilabel-indicator': [
-        np.random.RandomState(42).randint(2, size=(10, 10)),
-        np.array([[0, 1], [1, 0]]),
-        np.array([[0, 1], [1, 0]], dtype=np.bool),
-        np.array([[0, 1], [1, 0]], dtype=np.int8),
-        np.array([[0, 1], [1, 0]], dtype=np.uint8),
-        np.array([[0, 1], [1, 0]], dtype=np.float),
-        np.array([[0, 1], [1, 0]], dtype=np.float32),
-        np.array([[0, 0], [0, 0]]),
+        # valid when the data is formated as sparse or dense, identified
+        # by CSR format when the testing takes place
+        csr_matrix(np.random.RandomState(42).randint(2, size=(10, 10))),
+        csr_matrix(np.array([[0, 1], [1, 0]])),
+        csr_matrix(np.array([[0, 1], [1, 0]], dtype=np.bool)),
+        csr_matrix(np.array([[0, 1], [1, 0]], dtype=np.int8)),
+        csr_matrix(np.array([[0, 1], [1, 0]], dtype=np.uint8)),
+        csr_matrix(np.array([[0, 1], [1, 0]], dtype=np.float)),
+        csr_matrix(np.array([[0, 1], [1, 0]], dtype=np.float32)),
+        csr_matrix(np.array([[0, 0], [0, 0]])),
+        csr_matrix(np.array([[0, 1]])),
+        # Only valid when data is dense
         np.array([[-1, 1], [1, -1]]),
         np.array([[-3, 3], [3, -3]]),
-        np.array([[0, 1]]),
     ],
     'multilabel-sequences': [
         [[0, 1]],
@@ -51,6 +64,8 @@ EXAMPLES = {
         [0, 1, 2],
         ['a', 'b', 'c'],
         np.array([u'a', u'b', u'c']),
+        np.array([u'a', u'b', u'c'], dtype=object),
+        np.array(['a', 'b', 'c'], dtype=object),
     ],
     'multiclass-multioutput': [
         np.array([[1, 0, 2, 2], [1, 4, 2, 4]]),
@@ -60,6 +75,7 @@ EXAMPLES = {
         np.array([[1, 0, 2, 2], [1, 4, 2, 4]], dtype=np.float32),
         np.array([['a', 'b'], ['c', 'd']]),
         np.array([[u'a', u'b'], [u'c', u'd']]),
+        np.array([[u'a', u'b'], [u'c', u'd']], dtype=object),
         np.array([[1, 0, 2]]),
     ],
     'binary': [
@@ -79,7 +95,9 @@ EXAMPLES = {
         ['a'],
         ['a', 'b'],
         ['abc', 'def'],
+        np.array(['abc', 'def']),
         [u'a', u'b'],
+        np.array(['abc', 'def'], dtype=object),
     ],
     'continuous': [
         [1e-5],
@@ -127,10 +145,15 @@ def test_unique_labels():
     assert_array_equal(unique_labels([4, 0, 2]), np.array([0, 2, 4]))
 
     # Multilabels
-    assert_array_equal(unique_labels([(0, 1, 2), (0,), tuple(), (2, 1)]),
+    assert_array_equal(assert_warns(DeprecationWarning,
+                                    unique_labels,
+                                    [(0, 1, 2), (0,), tuple(), (2, 1)]),
                        np.arange(3))
-    assert_array_equal(unique_labels([[0, 1, 2], [0], list(), [2, 1]]),
+    assert_array_equal(assert_warns(DeprecationWarning,
+                                    unique_labels,
+                                    [[0, 1, 2], [0], list(), [2, 1]]),
                        np.arange(3))
+
     assert_array_equal(unique_labels(np.array([[0, 0, 1],
                                                [1, 0, 1],
                                                [0, 0, 0]])),
@@ -155,9 +178,16 @@ def test_unique_labels():
     # Some tests with strings input
     assert_array_equal(unique_labels(["a", "b", "c"], ["d"]),
                        ["a", "b", "c", "d"])
-    assert_array_equal(unique_labels([["a", "b"], ["c"]], [["d"]]),
+
+    assert_array_equal(assert_warns(DeprecationWarning, unique_labels,
+                                    [["a", "b"], ["c"]], [["d"]]),
                        ["a", "b", "c", "d"])
 
+
+@ignore_warnings
+def test_unique_labels_non_specific():
+    """Test unique_labels with a variety of collected examples"""
+
     # Smoke test for all supported format
     for format in ["binary", "multiclass", "multilabel-sequences",
                    "multilabel-indicator"]:
@@ -173,14 +203,17 @@ def test_unique_labels():
         for example in EXAMPLES[y_type]:
             assert_raises(ValueError, unique_labels, example)
 
-    #Mix of multilabel-indicator and multilabel-sequences
+
+@ignore_warnings
+def test_unique_labels_mixed_types():
+    # Mix of multilabel-indicator and multilabel-sequences
     mix_multilabel_format = product(EXAMPLES["multilabel-indicator"],
                                     EXAMPLES["multilabel-sequences"])
     for y_multilabel, y_multiclass in mix_multilabel_format:
         assert_raises(ValueError, unique_labels, y_multiclass, y_multilabel)
         assert_raises(ValueError, unique_labels, y_multilabel, y_multiclass)
 
-    #Mix with binary or multiclass and multilabel
+    # Mix with binary or multiclass and multilabel
     mix_clf_format = product(EXAMPLES["multilabel-indicator"] +
                              EXAMPLES["multilabel-sequences"],
                              EXAMPLES["multiclass"] +
@@ -202,6 +235,7 @@ def test_unique_labels():
                        ["0", "2"])
 
 
+@ignore_warnings
 def test_is_multilabel():
     for group, group_examples in iteritems(EXAMPLES):
         if group.startswith('multilabel'):
@@ -215,28 +249,61 @@ def test_is_multilabel():
 
 def test_is_label_indicator_matrix():
     for group, group_examples in iteritems(EXAMPLES):
-        if group == 'multilabel-indicator':
-            assert_, exp = assert_true, 'True'
+        if group in ['multilabel-indicator']:
+            dense_assert_, dense_exp = assert_true, 'True'
         else:
-            assert_, exp = assert_false, 'False'
+            dense_assert_, dense_exp = assert_false, 'False'
+
         for example in group_examples:
-            assert_(is_label_indicator_matrix(example),
-                    msg='is_label_indicator_matrix(%r) should be %s'
-                    % (example, exp))
+            # Only mark explicitly defined sparse examples as valid sparse
+            # multilabel-indicators
+            if group == 'multilabel-indicator' and issparse(example):
+                sparse_assert_, sparse_exp = assert_true, 'True'
+            else:
+                sparse_assert_, sparse_exp = assert_false, 'False'
+
+            if (issparse(example) or
+                (isinstance(example, np.ndarray) and
+                 example.ndim == 2 and
+                 example.dtype.kind in 'biuf' and
+                 example.shape[1] > 0)):
+                    examples_sparse = [sparse_matrix(example)
+                                       for sparse_matrix in [coo_matrix,
+                                                             csc_matrix,
+                                                             csr_matrix,
+                                                             dok_matrix,
+                                                             lil_matrix]]
+                    for exmpl_sparse in examples_sparse:
+                        sparse_assert_(is_label_indicator_matrix(exmpl_sparse),
+                                       msg=('is_label_indicator_matrix(%r)'
+                                       ' should be %s')
+                                       % (exmpl_sparse, sparse_exp))
+
+            # Densify sparse examples before testing
+            if issparse(example):
+                example = example.toarray()
+
+            dense_assert_(is_label_indicator_matrix(example),
+                          msg='is_label_indicator_matrix(%r) should be %s'
+                          % (example, dense_exp))
 
 
 def test_is_sequence_of_sequences():
     for group, group_examples in iteritems(EXAMPLES):
         if group == 'multilabel-sequences':
             assert_, exp = assert_true, 'True'
+            check = partial(assert_warns, DeprecationWarning,
+                            is_sequence_of_sequences)
         else:
             assert_, exp = assert_false, 'False'
+            check = is_sequence_of_sequences
         for example in group_examples:
-            assert_(is_sequence_of_sequences(example),
+            assert_(check(example),
                     msg='is_sequence_of_sequences(%r) should be %s'
                     % (example, exp))
 
 
+@ignore_warnings
 def test_type_of_target():
     for group, group_examples in iteritems(EXAMPLES):
         for example in group_examples:
@@ -246,3 +313,8 @@ def test_type_of_target():
 
     for example in NON_ARRAY_LIKE_EXAMPLES:
         assert_raises(ValueError, type_of_target, example)
+
+
+if __name__ == "__main__":
+    import nose
+    nose.runmodule()
